import { jsx as _jsx } from "react/jsx-runtime";
import { useState } from "react";
import Tooltip from "@mui/joy/Tooltip";
import Typography from "@mui/joy/Typography";
/**
 * NerdClickToCopy component
 *
 * @param {propsType} props Component props
 * @returns {ReactElement} NerdClickToCopy
 */
export default function NerdClickToCopy(props) {
    const { children, typographyProps, hoverBorderColor, hoverBackgroundColor } = props;
    const CLICK_TO_COPY = "Click to copy";
    const COPIED = "Copied !";
    const [tooltipText, setTooltipText] = useState(CLICK_TO_COPY);
    const sx = {
        ...typographyProps?.sx,
        margin: "1px 4px 1px 4px",
        padding: 0,
        ":hover": {
            backgroundColor: hoverBackgroundColor || "rgba(0, 153, 255, 0.28)",
            border: `1px ${hoverBorderColor || "rgba(0, 153, 255, 1)"} solid`,
            borderRadius: "5px",
            padding: "0px 3px 0px 3px",
            margin: 0,
        },
    };
    /**
     * Copy to clipboard
     *
     * @param {MouseEvent<HTMLSpanElement, MouseEvent>} e Click Event
     * @returns {Promise<void>}
     */
    async function copy(e) {
        if (!navigator.clipboard)
            return;
        const value = e.currentTarget.textContent;
        if (value !== null) {
            await navigator.clipboard.writeText(value);
            setTooltipText(COPIED);
        }
    }
    return (_jsx(Tooltip, { title: tooltipText, enterTouchDelay: 0, children: _jsx(Typography, { ...typographyProps, onClick: (e) => {
                typographyProps?.onClick?.(e);
                copy(e);
            }, onMouseOver: () => setTooltipText(CLICK_TO_COPY), sx: sx, children: children }) }));
}
