import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Avatar, 
// Badge,
Box, Checkbox, Chip, 
// ChipDelete,
Divider, Grid, IconButton, List, ListItem, ListItemButton, ListItemDecorator, Sheet, Tooltip, } from "@mui/joy";
import { Drawer } from "@mui/material";
// import { Popper, ClickAwayListener } from "@mui/base";
import { useLocalStorage } from "@uidotdev/usehooks";
import { DateTime } from "luxon";
import { useCallback, useEffect, useMemo, 
// useRef,
useState, } from "react";
import { useDispatch } from "react-redux";
import { Link } from "react-router-dom";
import { useLocation } from "react-router-dom";
import { nerdHistoryOpenSelector, setNerdHistoryOpen, setNerdHistoryOpenKey, } from "../redux/nerdHistory";
import { useSelector } from "react-redux";
// import { arrayToMap } from "@nerdjs/nerd-core";
import { useTranslation } from "react-i18next";
/**
 *
 * @param {NerdHistoryProps} props Props
 * @returns {ReactElement} NerdHistory
 */
export function NerdHistory(props) {
    const { historykey, openKey, anchor = "right", pathesToIgnore,
    // appID,
    // apps: _apps,
     } = props;
    const { t } = useTranslation();
    const location = useLocation();
    const [_history, saveHistory] = useLocalStorage(historykey, []);
    const [selectMode, setSelectMode] = useState(false);
    const [selectedItems, setSelectedItems] = useState(new Set());
    const [allSelected, setAllSelected] = useState(false);
    const open = useSelector(nerdHistoryOpenSelector);
    const dispatch = useDispatch();
    // const [openFilters, setOpenFilters] = useState(false);
    // const filterChipRef = useRef<HTMLDivElement>(null);
    // const [filters, setFilters] = useState<Set<number>>(new Set());
    useEffect(() => {
        saveHistory((history) => {
            const test = history.filter((h) => {
                const date = DateTime.fromMillis(h.date);
                if (DateTime.now().diff(date, "days").days >= 7)
                    return false;
                else
                    return true;
            });
            return test;
        });
    }, []);
    useEffect(() => {
        dispatch(setNerdHistoryOpenKey(openKey));
    }, [openKey]);
    useEffect(() => {
        const _open = localStorage.getItem(openKey);
        dispatch(setNerdHistoryOpen(_open == "true" ? true : false));
    }, []);
    useEffect(() => {
        let match = false;
        if (pathesToIgnore)
            for (let i = 0; i < pathesToIgnore.length; i++) {
                const rx = pathesToIgnore[i];
                match = rx.test(location.pathname);
                if (match)
                    break;
            }
        if (!match)
            setTimeout(() => {
                const index = _history.findIndex((h) => h.endpoint === window.location.origin + location.pathname);
                if (index !== -1) {
                    const newHistory = [..._history];
                    newHistory[index] = {
                        date: DateTime.now().toMillis(),
                        endpoint: window.location.origin + location.pathname,
                        label: document.title,
                        subtitle: location.state?.subtitle || "",
                        // appID,
                        pinned: newHistory[index].pinned,
                    };
                    saveHistory(newHistory);
                }
                else
                    saveHistory([
                        ..._history,
                        {
                            date: DateTime.now().toMillis(),
                            endpoint: window.location.origin + location.pathname,
                            label: document.title,
                            subtitle: location.state?.subtitle || "",
                            // appID,
                            pinned: false,
                        },
                    ]);
            }, 100);
    }, [location]);
    const onClose = () => {
        dispatch(setNerdHistoryOpen(false));
    };
    // const apps = useMemo(() => {
    //   return arrayToMap(_apps);
    // }, [_apps]);
    // const sortedApps = useMemo(() => {
    //   return [..._apps].sort((a, b) =>
    //     a.name && b.name
    //       ? a.name?.localeCompare(b.name)
    //       : a.name
    //       ? 1
    //       : b.name
    //       ? -1
    //       : 0
    //   );
    // }, [_apps]);
    const historyPinned = useMemo(() => {
        // return filters.size > 0
        //   ? _history
        //       .filter((h) => filters.has(h.appID) && h.pinned)
        //       .sort((a, b) => b.date - a.date)
        //   : _history.filter((h) => h.pinned).sort((a, b) => b.date - a.date);
        return _history.filter((h) => h.pinned).sort((a, b) => b.date - a.date);
    }, [
        _history,
        // filters
    ]);
    const historyUnPinned = useMemo(() => {
        // return filters.size > 0
        //   ? _history
        //       .filter((h) => filters.has(h.appID) && !h.pinned)
        //       .sort((a, b) => b.date - a.date)
        //   : _history.filter((h) => !h.pinned).sort((a, b) => b.date - a.date);
        return _history.filter((h) => !h.pinned).sort((a, b) => b.date - a.date);
    }, [
        _history,
        // filters
    ]);
    const pinOrUnpinItem = useCallback((item) => {
        const index = _history.findIndex((h) => h.endpoint === item.endpoint);
        if (index !== -1) {
            const newHistory = [..._history];
            newHistory[index] = {
                ...newHistory[index],
                pinned: !newHistory[index].pinned,
            };
            saveHistory(newHistory);
        }
    }, [_history]);
    return (_jsx(_Fragment, { children: _jsx(Drawer, { open: open, onClose: onClose, variant: "persistent", sx: {
                width: 250,
                flexShrink: 0,
                "& .MuiDrawer-paper": {
                    width: 250,
                },
            }, anchor: anchor, children: _jsxs(Box, { sx: {
                    display: "flex",
                    gap: 1,
                    alignItems: "stretch",
                    flexDirection: "column",
                    mt: { xs: "140px", sm: "100px" },
                }, children: [_jsx(Sheet, { sx: {
                            p: 0.5,
                        }, children: _jsxs(Grid, { container: true, spacing: 1, alignItems: "start", children: [selectMode ? (_jsxs(_Fragment, { children: [_jsx(Grid, { children: _jsx(Chip, { disabled: _history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                                    if (!allSelected) {
                                                        setSelectedItems(new Set(_history.map((h) => h.date)));
                                                    }
                                                    else
                                                        setSelectedItems(new Set());
                                                    setAllSelected((s) => !s);
                                                }, children: allSelected ? "Unselect All" : "Select All" }) }), _jsx(Grid, { children: _jsx(Chip, { size: "sm", variant: "soft", onClick: () => {
                                                    setSelectMode(false);
                                                    setAllSelected(false);
                                                    setSelectedItems(new Set());
                                                }, children: t("common:global.cancel") }) }), _jsx(Grid, { ml: "auto", children: _jsx(Tooltip, { title: "Clear History", children: _jsx(IconButton, { disabled: selectedItems.size === 0, size: "sm", color: "danger", variant: "soft", onClick: () => {
                                                        saveHistory(_history.filter((h) => !selectedItems.has(h.date)));
                                                        setSelectedItems(new Set());
                                                        setAllSelected(false);
                                                    }, children: _jsx("i", { className: "fa-solid fa-trash" }) }) }) })] })) : (_jsx(_Fragment, { children: _jsx(Grid, { ml: "auto", children: _jsx(Chip, { disabled: _history.length === 0, size: "sm", variant: "soft", color: "primary", onClick: () => {
                                                setSelectMode(true);
                                            }, children: "Select" }) }) })), _jsx(Grid, { children: _jsx(Tooltip, { title: "Close", children: _jsx(IconButton, { size: "sm", variant: "plain", onClick: onClose, children: _jsx("i", { className: "fa-solid fa-xmark" }) }) }) })] }) }), historyPinned.length > 0 ? (_jsxs(Box, { width: "100%", height: "100%", children: [_jsx(List, { size: "sm", children: historyPinned.map((h, i) => (
                                // filters.size == 0 || filters.has(h.appID) ? (
                                _jsxs(ListItem, { children: [selectMode ? (_jsx(ListItemDecorator, { children: _jsx(Checkbox, { size: "sm", checked: selectedItems.has(h.date) || false, onChange: (e) => {
                                                    const checked = e.target.checked;
                                                    setSelectedItems((state) => {
                                                        const newState = new Set(state);
                                                        if (checked)
                                                            newState.add(h.date);
                                                        else
                                                            newState.delete(h.date);
                                                        return newState;
                                                    });
                                                    if (!checked) {
                                                        setAllSelected(false);
                                                    }
                                                } }) })) : (_jsx(IconButton, { size: "sm", onClick: () => pinOrUnpinItem(h), color: "danger", children: _jsx("i", { className: "fa-solid fa-thumbtack" }) })), _jsx(Tooltip, { title: h.subtitle || "", children: _jsx(ListItemButton, { component: Link, to: h.endpoint, sx: {
                                                    textDecoration: "none !important",
                                                }, children: _jsx(Box, { display: "flex", gap: 1, height: "100%", width: "100%", children: h.label }) }) })] }, i))
                                // ) : null
                                ) }), _jsx(Divider, {})] })) : null, _jsx(Box, { width: "100%", height: "100%", children: historyUnPinned.length > 0 ? (_jsx(List, { size: "sm", children: historyUnPinned.map((h, i) => (
                            // filters.size == 0 || filters.has(h.appID) ? (
                            _jsxs(ListItem, { sx: {
                                    "& .pin-button": {
                                        visibility: "hidden",
                                    },
                                    "&:hover": {
                                        "& .pin-button": {
                                            visibility: "visible",
                                        },
                                    },
                                }, children: [selectMode ? (_jsx(ListItemDecorator, { children: _jsx(Checkbox, { size: "sm", checked: selectedItems.has(h.date) || false, onChange: (e) => {
                                                const checked = e.target.checked;
                                                setSelectedItems((state) => {
                                                    const newState = new Set(state);
                                                    if (checked)
                                                        newState.add(h.date);
                                                    else
                                                        newState.delete(h.date);
                                                    return newState;
                                                });
                                                if (!checked) {
                                                    setAllSelected(false);
                                                }
                                            } }) })) : (_jsx(IconButton, { size: "sm", className: "pin-button", onClick: () => pinOrUnpinItem(h), children: _jsx("i", { className: "fa-solid fa-thumbtack" }) })), _jsx(Tooltip, { title: h.subtitle || "", children: _jsx(ListItemButton, { component: Link, to: h.endpoint, sx: {
                                                textDecoration: "none !important",
                                            }, children: _jsx(Box, { display: "flex", gap: 1, height: "100%", width: "100%", children: h.label }) }) })] }, i))
                            // ) : null
                            ) })) : (_jsx(Box, { display: "flex", justifyContent: "center", width: "100%", height: "100%", children: _jsx(Avatar, { size: "lg", sx: {
                                    mt: 2,
                                }, children: _jsx("i", { className: "fa-regular fa-cactus" }) }) })) })] }) }) }));
}
